package gonin.renevier.philippe.exempletest;


import java.util.concurrent.TimeUnit;

import com.robotium.solo.Solo;

import android.app.Instrumentation;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.test.ActivityInstrumentationTestCase2;
import android.test.UiThreadTest;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.TextView;

import static gonin.renevier.philippe.exempletest.R.id;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class TestApplication extends ActivityInstrumentationTestCase2<TodoList> {

    TodoList todo;
    EditText nouveau;
    Button ajouter;
    LinearLayout liste;

    private Solo mSolo;


    // pour les tests
    String [] taches = { "nouvelle tache 1", "nouvelle tache 2", "nouvelle tache 3" };


    public TestApplication() {
        this(TodoList.class);
    }
    public TestApplication(Class<TodoList> activityClass) {
        super(activityClass);
        // TODO Auto-generated constructor stub
    }

    protected void setUp() throws Exception {
        super.setUp();


        /************* mock de sauvegarde, pour ne pas charger des notes **************/
        Instrumentation instru =  getInstrumentation();
        Context c = instru.getTargetContext();

        System.setProperty( "dexmaker.dexcache",c.getCacheDir().getPath());

        Sauvegarde mockSauvegarde = mock(Sauvegarde.class);
        Sauvegarde.Donnees[] tabVide = new Sauvegarde.Donnees[0];
        when(mockSauvegarde.charger(any(Context.class))).thenReturn(tabVide);
        TodoList.sauvegarde = mockSauvegarde;
        /************************** fin du mock de sauvegarde *************************/

        todo = getActivity();



        nouveau = (EditText) todo.findViewById(id.nouveau);
        ajouter = (Button) todo.findViewById(id.ajouter);
        liste = (LinearLayout) todo.findViewById(id.liste);


        mSolo = new Solo(getInstrumentation(), getActivity());


    }


    protected void tearDown() throws Exception {
        super.tearDown();
    }


    private Runnable clickOnView(final View v) {
        return new Runnable() {
            public void run() {
                v.callOnClick();
            }
        };
    }

    private void runOnUI(Runnable r) {
        todo.runOnUiThread(r);
        getInstrumentation().waitForIdleSync();
    }

    private View[] getChilds(ViewGroup v) {
        View [] res = null;
        if (v != null) {
            res = new View[v.getChildCount()];
            for(int i = 0; i < res.length; i++) res[i] = v.getChildAt(i);
        }
        return res;
    }

    public void testAjout() {
        // focus sur le edittext, normalement par defaut... mais
        runOnUI(
                new Runnable() {
                    public void run() {
                        nouveau.requestFocus();
                    }
                }
        );


        for (int i = 0; i  < taches.length; i++) {
            getInstrumentation().sendStringSync(taches[i]);
            // ajouter.callOnClick(); // provoque android.view.ViewRootImpl$CalledFromWrongThreadException: Only the original thread that created a view hierarchy can touch its views.

            runOnUI(clickOnView(ajouter));
            assertEquals("nombre de vue sous liste", i+1, liste.getChildCount());

            try {
                TimeUnit.SECONDS.sleep(2);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

        }

    }

    @UiThreadTest
    public void testAjoutOnUITheard() {

        String [] taches = { "nouvelle tache 1", "nouvelle tache 2", "nouvelle tache 3" };

        // focus sur le edittext, normalement par defaut... mais
        // nouveau.requestFocus();


        for (int i = 0; i  < taches.length; i++) {
            nouveau.setText(taches[i]);
            // moniteur.sendStringSync(taches[i]); // provoque java.lang.RuntimeException: This method can not be called from the main application thread

            ajouter.callOnClick(); // empiler un event... dans le même thread...
            // du coup fait apres...

            // assertEquals("nombre de vue sous liste", i+1, liste.getChildCount());
            // cet assert ne peut pas être verifier...
        }

    }


    public void testChangement() {
        testAjout();

        // on recupere les Item
        View [] items = getChilds(liste);
        int nb_items = items.length;

        // on memorise le premier apres les ajouts... qui sera deplace en dernier en n-1 etapes
        View first = items[0];
        ImageButton down = (ImageButton) first.findViewWithTag("down");

        for(int i = 0; i < nb_items-1; i++) {
            assertEquals("nombre de vue sous liste", taches.length, items.length);
            // on deplace le 1er au dernier
            runOnUI(clickOnView(down));
            // on recupere a nouveau les Item
            items = getChilds(liste);
            assertEquals("l'item est en "+(i+1), items[i+1], first);

            try {
                TimeUnit.SECONDS.sleep(2);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }

    }


	/** /
	public void testRotation_naif() {

		testChangement();

		// on recupere les Item
		View [] items = getChilds(liste);
		int nb_items = items.length;

		assertEquals("nombre de vue sous liste", taches.length, items.length);
		View last_wasFirst = items[nb_items-1];

		todo.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);



		liste = (LinearLayout) todo.findViewById(id.liste);
		// renvoie toujours la premiere activité qui a ete detruit par la rotation...
		// getActivity() aussi


		items = getChilds(liste);

		assertEquals("nombre de vue sous liste", taches.length, items.length);
		assertEquals("nombre inchange de vue sous liste", nb_items, items.length);
		assertEquals("l'item est en "+nb_items, items[nb_items-1], last_wasFirst);

		todo.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

        // temps pour android de detruire les refs //
        try {
            TimeUnit.SECONDS.sleep(5);
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

		 liste = (LinearLayout) todo.findViewById(id.liste);


		items = getChilds(liste);

		assertEquals("nombre de vue sous liste", taches.length, items.length);
		assertEquals("nombre inchange de vue sous liste ", nb_items, items.length);
		assertEquals("l'item est en "+nb_items, items[nb_items-1], last_wasFirst);



	}
    / **/





    public void testRotation() {

        int orientation = todo.getRequestedOrientation();

        testChangement();

        // on recupere les Item
        View [] items = getChilds(liste);
        int nb_items = items.length;

        assertEquals("nombre de vue sous liste", taches.length, items.length);
        // en v4 il n'ont pas de tag.. seul le texte...
        // en v7 le tag, ca passe...
        Object last_wasFirst = items[nb_items-1].getTag();
        String texte = ((TextView) (items[nb_items-1].findViewWithTag("note"))).getText().toString();


        if (orientation == ActivityInfo.SCREEN_ORIENTATION_PORTRAIT) mSolo.setActivityOrientation(Solo.LANDSCAPE);
        else mSolo.setActivityOrientation(Solo.PORTRAIT);

        mSolo.waitForActivity(TodoList.class);
        getInstrumentation().waitForIdleSync();


        todo = (TodoList) mSolo.getCurrentActivity(); //this will return new activity
        liste = (LinearLayout) todo.findViewById(id.liste);


        items = getChilds(liste);

        System.out.println(last_wasFirst);

        assertEquals("nombre de vue sous liste "+liste.getChildCount(), taches.length, items.length);
        assertEquals("nombre inchange de vue sous liste", nb_items, items.length);
        // les obj sont differents... mais les tags ont du etre sauves
        assertEquals("l'item est en "+nb_items, items[nb_items-1].getTag(), last_wasFirst);
        String texte2 = ((TextView) (items[nb_items-1].findViewWithTag("note"))).getText().toString();
        assertEquals("vraiment le meme ? ", texte, texte2);


        // encore une rotation

        if (orientation == ActivityInfo.SCREEN_ORIENTATION_PORTRAIT) mSolo.setActivityOrientation(Solo.PORTRAIT);
        else mSolo.setActivityOrientation(Solo.LANDSCAPE);

        mSolo.waitForActivity(TodoList.class);
        getInstrumentation().waitForIdleSync();

        todo = (TodoList) mSolo.getCurrentActivity(); //this will return new activity

        liste = (LinearLayout) todo.findViewById(id.liste);


        items = getChilds(liste);

        assertEquals("nombre de vue sous liste", taches.length, items.length);
        assertEquals("nombre inchange de vue sous liste ", nb_items, items.length);
        assertEquals("l'item est en "+nb_items, items[nb_items-1].getTag(), last_wasFirst);
        texte2 = ((TextView) (items[nb_items-1].findViewWithTag("note"))).getText().toString();
        assertEquals("vraiment le meme ? ", texte, texte2);

        // sans cela + le test sur le texte, le piege du tag null peut faire passer un test qui ne devrait pas
        assertNotNull("il doit y avoir un tag...", last_wasFirst);

    }

}