package gonin.renevier.philippe.exempletest;

import android.content.Context;
import android.content.res.AssetFileDescriptor;
import android.test.AndroidTestCase;
import android.test.mock.MockContext;


import org.mockito.Matchers;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Arrays;

import static org.mockito.Matchers.anyInt;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.mock;


/**
 * Created by Philippe on 12/02/2015.
 */
public class TestSauvergarde extends AndroidTestCase {


    Context initialContext;

    /** oracle **/
    String [] notes = { "txt = une dernière || dateButtoir = 1424646313644 || dateCreation = 1423695934085" ,
            "txt = toto || dateButtoir = 1423609723596 || dateCreation = 1423609729089" ,
            "txt = yuip || dateButtoir = 1425683388340 || dateCreation = 1423609796536" ,
            "txt = azerty || dateButtoir = 1424560229853 || dateCreation = 1423696242824"};

    Long [] buttoirs = {1424646313644L, 1423609723596L, 1425683388340L, 1424560229853L};
    Long [] creations = {1423695934085L, 1423609729089L, 1423609796536L, 1423696242824L};



    protected class MyMockContext extends MockContext {
        /** the file must not be compressed... **/
        int idRaw;
        String written = "";

        MyMockContext() {
            this(R.raw.storage_input_test);
        }


        MyMockContext(int idRaw) {
               super();
            this.idRaw = idRaw; // "android.resource://gonin.renevier.philippe.exempletest/raw/"+fileNameInRaw;
        }

        public FileInputStream openFileInput(String name) {
            AssetFileDescriptor afd = initialContext.getResources().openRawResourceFd(idRaw);
            FileInputStream res = null;

            res =  new FileInputStream(afd.getFileDescriptor());
            return res;
        }

        @Override
        public FileOutputStream openFileOutput(String name, int mode) {
            FileOutputStream res = mock(FileOutputStream.class);
            try {
                doAnswer(new Answer() {
                    public Object answer(InvocationOnMock invocation) {
                        Object[] args = invocation.getArguments();
                        byte b = ((Byte) args[0]).byteValue();
                        written = written + Byte.toString(b);
                        return null;
                    }}).when(res).write(anyInt());

                doAnswer(new Answer() {
                    public Object answer(InvocationOnMock invocation) {
                        Object[] args = invocation.getArguments();
                        byte[] b = (byte[]) args[0];
                        try {
                            written = written + new String(b, "UTF-8");
                        } catch (UnsupportedEncodingException e) {
                            e.printStackTrace();
                        }
                        return null;
                    }}).when(res).write(Matchers.<byte[]>any());

                doAnswer(new Answer() {
                    public Object answer(InvocationOnMock invocation) {
                        Object[] args = invocation.getArguments();
                        byte[] bytes = (byte[]) args[0];
                        int start = ((Integer) args[1]).intValue();
                        int len = ((Integer) args[2]).intValue();
                        /*
                        for(int i = start; i < start+len; i++) {
                            if (i < bytes.length) {
                                written = written + (char) bytes[i];
                            }
                        }
                        */
                        bytes = Arrays.copyOfRange(bytes, start, start+len);
                        try {
                            written = written + new String(bytes, "UTF-8");
                        } catch (UnsupportedEncodingException e) {
                            e.printStackTrace();
                        }
                        return null;
                    }}).when(res).write(Matchers.<byte[]>any(), anyInt(), anyInt());

            } catch (IOException e) {
                e.printStackTrace();
            }

            return res;
        }
    }


    protected void setUp() throws Exception {
        super.setUp();

        initialContext = getContext();

        System.setProperty(
                "dexmaker.dexcache",
                initialContext.getCacheDir().getPath());
    }

    public void testLoad() {
        // org.mockito.
        MyMockContext context = new MyMockContext(R.raw.storage_input_test);
        setContext(context);

        Sauvegarde load =  Sauvegarde.getInstance();
        Sauvegarde.Donnees[] res = load.charger(context);
        assertEquals("non vide", 4, res.length);

        for(int i = 0; i < res.length; i++) {
            assertEquals("meme texte", notes[i], res[i].note);
            assertEquals("meme date buttoire", (long) buttoirs[i], res[i].dateButtoir);
            assertEquals("meme date creation", (long) creations[i], res[i].dateCreation);
        }


    }



    public void testStore() {
        MyMockContext context = new MyMockContext();
        setContext(context);

        ArrayList<Item> listItem = new ArrayList<Item>();
        for(int i = 0; i < notes.length; i++) {
            Item item = mock(Item.class);
            doReturn(notes[i]).when(item).getNote();
            doReturn(buttoirs[i]).when(item).getButtoir();
            doReturn(creations[i]).when(item).getCreation();
            listItem.add(item);
        }

        Sauvegarde store = Sauvegarde.getInstance();
        store.sauvegarder(listItem, context);

        String expected = "/* saved to do list */\n" +
                "txt = une dernière || dateButtoir = 1424646313644 || dateCreation = 1423695934085#;#1424646313644#;#1423695934085\n" +
                "txt = toto || dateButtoir = 1423609723596 || dateCreation = 1423609729089#;#1423609723596#;#1423609729089\n" +
                "txt = yuip || dateButtoir = 1425683388340 || dateCreation = 1423609796536#;#1425683388340#;#1423609796536\n" +
                "txt = azerty || dateButtoir = 1424560229853 || dateCreation = 1423696242824#;#1424560229853#;#1423696242824\n";

        assertEquals("donnees ecrites prevues...", expected, context.written);
    }
}
